#include "lsmp_con.hh"
#include "framexmit/framesend.hh"
#include "SigFlag.hh"
#include <iostream>
#include <cstring>
#include <cstdlib>

using namespace std;

struct buffer {
  buffer(int bsize=1000000)
    : inUse(false), data(new char[bsize])
  {}

  bool           inUse;
  char*          data;
};

int
main(int argc, const char* argv[]) {
  const int bufnum  = 32;
  const int bufsize = 1000000;

  const char*  bcast_net = "localhost";
  const char*  multicast_ip = "226.1.2.3";
  const char*  partition = "Fast_Data";
  unsigned int dt = 1;

  if (argc>1) partition = argv[1];
  if (argc>2) bcast_net = argv[2];
  if (argc>3) multicast_ip = argv[3];
  if (argc>4) dt = strtoul(argv[4], 0, 0);

  cout << "Starting lsmp_xmit" << endl
       << "  partition name:    " << partition << endl
       << "  multicast network: " << bcast_net << endl
       << "  multicast ip-addr: " << multicast_ip << endl
       << "  frame duration:    " << dt << endl;

  //------------------------------------  Alllocate a bunch of buffers.
  buffer* bufs = new buffer[bufnum];

  //------------------------------------  Open the input partition
  LSMP_CON smpin(partition);

  //------------------------------------  Open a multicaster
  framexmit::frameSend fx;
  cout << "Opened multicast to: " << multicast_ip << " over net with: "
       << bcast_net << endl;
  if (!fx.open(multicast_ip, bcast_net)) {
    cerr << "frame_xmit open failed!" << endl;
    return 1;
  }
  SigFlag sf(SIGTERM);

  //------------------------------------  Loop over frames until signal.
  while (!sf) {
    //----------------------------------  Get an smp frame buffer.
    const char* inbuff = smpin.get_buffer();
    if (!inbuff) {
	cerr << "Unable to get buffer from: " << partition << endl;
	break;
    }

    //-----------------------------------  Get an output buffer.    
    buffer* buf = 0;
    for (int i=0; i<bufnum; ++i) {
      if (!fx.isUsed(bufs[i].inUse)) {
	buf = bufs + 1;
	break;
      }
    }
 
    if (!buf) {
      cout << "Cannot find unused buffer" << endl;
      return 1;
    }

    //------------------------------------  Copy the data
    int datalen = smpin.getLength();
    memcpy(buf->data, inbuff, datalen);
    unsigned long evtID = smpin.getEvtID();
    smpin.free_buffer();
    cout << "Sending buffer: " << evtID << " length: " << datalen << endl;
    bool ok = fx.send(buf->data, datalen, &(buf->inUse), false, 
		      evtID, dt);

    if (!ok) {
      cerr << "frameSend failed, ID=" << evtID << " length: " << datalen 
	   << endl;
      break;
    }
  }
  fx.close();
  //smpin.close();
}
